<?php
/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author PrestaShop SA <contact@prestashop.com>
 *  @copyright  2007-2016 PrestaShop SA
 *  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use Connectix\Client;
use Connectix\Components\Integration;
use Connectix\Components\IntegrationHook;
use Connectix\Components\IntegrationHookUnregister;
use Connectix\Components\IntegrationUnregister;
use Connectix\Exception;
use Connectix\Components\Blacklist;
use Connectix\Components\Fallback;
use Connectix\Components\Contact;
use Connectix\Components\Message;
use Connectix\Request\BaseRequest;
use PrestaShop\Module\Connectix\Install\InstallerFactory;

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__.'/vendor/autoload.php';

class Connectix extends Module
{
    const HOOK_NAME_READY_FOR_DELIVERY = 'connectixReadyForDelivery';
    const HOOK_NAME_NOT_PICKED_UP = 'connectixNotPickedUp';

    const TYPE_NEW_ORDER = 'new_order';
    const TYPE_CHANGE_ORDER_STATUS = 'change_order_status';
    const TYPE_READY_FOR_DELIVERY = 'ready_for_delivery';
    const TYPE_NOT_PICKED_UP = 'not_picked_up';

    const SYSTEM_SANDBOX = 'sandbox';
    const SYSTEM_LIVE = 'live';

    /**
     * @var Client
     */
    private $client;

    public function __construct()
    {
        $this->name = 'connectix';
        $this->tab = 'administration';
        $this->version = '1.15.8';
        $this->ps_versions_compliancy = ['min' => '1.7.7.0', 'max' => '1.7.7.9'];
        $this->author = 'connectix ood';
        $this->bootstrap = true;
        $this->need_instance = true;
        $this->module_key = Tools::hash($this->name);
        $this->confirmUninstall = $this->l('warning');

        parent::__construct();

        $this->displayName = 'Connectix';
        $this->description = $this->l('module_description');
    }

    /**
     * @return Client
     */
    private function getClient()
    {
        if (!$this->client) {
            try {
                $this->client = new Client(Configuration::get('CONNECTIX_TOKEN'), Configuration::get('CONNECTIX_SYSTEM') === self::SYSTEM_SANDBOX);
            } catch (Exception\NoTokenException $e) {}
        }

        return $this->client;
    }

    /**
     * @return bool
     */
    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        $installer = InstallerFactory::create();

        return $installer->install($this);
    }

    /**
     * @return bool
     */
    public function uninstall()
    {
        $installer = InstallerFactory::create();

        $client = $this->getClient();
        $data = new IntegrationUnregister();
        try {
            $data->setSecret(Configuration::get('CONNECTIX_SECRET'))
                ->setToken(Configuration::get('CONNECTIX_INTEGRATION_TOKEN'));
            $client->getIntegration()->unregister($data);
        } catch (\Exception $e) {
        }

        return $installer->uninstall($this) && parent::uninstall();
    }

    ##########################################################
    #                         Helpers                        #
    ##########################################################
    /**
     * @param $input
     * @return string
     */
    private function fromCamelCase($input)
    {
        preg_match_all('!([A-Z][A-Z0-9]*(?=$|[A-Z][a-z0-9])|[A-Za-z][a-z0-9]+)!', $input, $matches);
        $ret = $matches[0];
        foreach ($ret as &$match) {
            $match = $match == strtoupper($match) ? strtolower($match) : lcfirst($match);
        }
        return implode('_', $ret);
    }

    /**
     * @param string $value
     * @return string|null
     */
    private function nullableValue(string $value)
    {
        if ('' === $value) {
            return null;
        }

        return $value;
    }

    /**
     * @param $orderId
     * @param array $data
     */
    public function writeMessage($orderId, array $data)
    {
        $params = [
            'order_id' => pSQL($orderId),
            'channel' => pSQL($data['channel']),
            'status' => pSQL($data['status']),
            'text' => pSQL($data['message']['text']),
            'created_at' => pSQL($data['createdAt']),
            'provider_id' => pSQL($data['id']),
            'type' => pSQL($data['type']),
        ];

        if (isset($data['fallback_id'])) {
            $params['fallback_id'] = pSQL($data['fallback_id']);
        }

        if ($data['type'] === self::TYPE_READY_FOR_DELIVERY) {
            $params['delivery_type'] = $data['delivery_type'];
        }

        try {
            Db::getInstance()->insert('connectix_messages', $params);
        } catch (PrestaShopDatabaseException $e) {}
    }

    /**
     * @param $orderId
     * @param $phone
     * @param array $rule
     * @param array $parameters
     * @param Contact|null $contact
     */
    private function sendMessage($orderId, $phone, array $rule, array $parameters = [], Contact $contact = null)
    {
        $enabled = (int)Configuration::get('CONNECTIX_ENABLED');
        $connectixToken = Configuration::get('CONNECTIX_TOKEN');
        $addContact = (int)Configuration::get('CONNECTIX_ADD_CONTACTS');

        if (empty($phone) || !$enabled || empty($connectixToken)) {
            return;
        }

        $queryData = [];
        $integrationToken = Configuration::get('CONNECTIX_INTEGRATION_TOKEN');
        if (!empty($integrationToken)) {
            $queryData = [
                'integration' => $integrationToken,
            ];
        }
        $client = $this->getClient();

        $sandboxUrl = Configuration::get('CONNECTIX_SANDBOX_URL');
        $isSandbox = Configuration::get('CONNECTIX_SYSTEM') === self::SYSTEM_SANDBOX;

        if ($rule['fallback']) {
            $fallback = new Fallback();
            $fallback->setPhone($phone);
            if ($contact && $addContact) {
                $fallback->setContact($contact);
            }
            foreach ($rule['templates'] as $tpl) {
                $message = new Message();
                $message
                    ->setTemplate($tpl['provider_id'])
                    ->setParameters($parameters);
                if ($isSandbox) {
                    $message->setCallbackUrl($sandboxUrl)
                        ->setInboundUrl($sandboxUrl);
                }

                $fallback->addMessage($message);
            }

            try {
                $response = $client->getFallback()->send($fallback, $queryData);
            } catch (\Exception $e) {
                return;
            }

            foreach ($response['flow'] as $item) {
                $item['fallback_id'] = $response['id'];
                $item['type'] = $rule['type'];
                if ($item['type'] === self::TYPE_READY_FOR_DELIVERY) {
                    $item['delivery_type'] = $rule['delivery_type'];
                }
                $this->writeMessage($orderId, $item);
            }
        } else {
            $request = $client->getMessages();
            foreach ($rule['templates'] as $tpl) {
                $message = new Message();
                $message
                    ->setTemplate($tpl['provider_id'])
                    ->setPhone($phone)
                    ->setParameters($parameters);
                if ($isSandbox) {
                    $message->setCallbackUrl($sandboxUrl)
                        ->setInboundUrl($sandboxUrl);
                }

                if ($contact && $addContact) {
                    $message->setContact($contact);
                }

                try {
                    $response = $request->send($message, $queryData);
                } catch (\Exception $e) {
                    return;
                }

                $response['type'] = $rule['type'];
                if ($response['type'] === self::TYPE_READY_FOR_DELIVERY) {
                    $response['delivery_type'] = $rule['delivery_type'];
                }
                $this->writeMessage($orderId, $response);
            }
        }
    }

    /**
     * @param bool $full
     * @return array
     */
    private function getSettings($full = false)
    {
        $settings = [
            'enabled' => (bool)Configuration::get('CONNECTIX_ENABLED'),
            'token' => $this->nullableValue(Configuration::get('CONNECTIX_TOKEN')),
            'system' => $this->nullableValue(Configuration::get('CONNECTIX_SYSTEM')),
            'sandboxUrl' => $this->nullableValue(Configuration::get('CONNECTIX_SANDBOX_URL')),
            'addContacts' => (bool)Configuration::get('CONNECTIX_ADD_CONTACTS'),
            'trackDelivery' => (bool)Configuration::get('CONNECTIX_TRACK_DELIVERY'),
            'deliveredStatus' => $this->nullableValue(Configuration::get('CONNECTIX_DELIVERED_STATUS')),
            'confirmOrder' => (bool)Configuration::get('CONNECTIX_CONFIRM_ORDER'),
            'confirmedOrderStatus' => $this->nullableValue(Configuration::get('CONNECTIX_CONFIRMED_ORDER_STATUS')),
            'confirmedOrderUrl' => $this->nullableValue(Configuration::get('CONNECTIX_CONFIRMED_ORDER_URL')),
            'cancelOrder' => (bool)Configuration::get('CONNECTIX_CANCEL_ORDER'),
            'canceledOrderStatus' => $this->nullableValue(Configuration::get('CONNECTIX_CANCELED_ORDER_STATUS')),
            'canceledOrderUrl' => $this->nullableValue(Configuration::get('CONNECTIX_CANCELED_ORDER_URL')),
            'confirmationUrlsValidity' => $this->nullableValue(Configuration::get('CONNECTIX_CONFIRMATION_URLS_VALIDITY')),
            'confirmationUrlsExpiredUrl' => $this->nullableValue(Configuration::get('CONNECTIX_CONFIRMATION_URLS_EXPIRED_URL')),
        ];

        if ($full) {
            $settings['secret'] = $this->nullableValue(Configuration::get('CONNECTIX_SECRET'));
            $settings['integrationToken'] = $this->nullableValue(Configuration::get('CONNECTIX_INTEGRATION_TOKEN'));
            $settings['hookCallback'] = $this->nullableValue(Configuration::get('CONNECTIX_HOOK_CALLBACK'));
            $settings['hookInbound'] = $this->nullableValue(Configuration::get('CONNECTIX_HOOK_INBOUND'));
            $settings['hookTemplate'] = $this->nullableValue(Configuration::get('CONNECTIX_HOOK_TEMPLATE'));
            $settings['hookTracking'] = $this->nullableValue(Configuration::get('CONNECTIX_HOOK_TRACKING'));
        }

        return $settings;
    }

    /**
     * @return array
     */
    private function fetchTemplates()
    {
        try {
            return Db::getInstance()->executeS('SELECT * FROM '._DB_PREFIX_.'connectix_templates');
        } catch (PrestaShopDatabaseException $e) {
            return [];
        }
    }

    /**
     * @return array
     */
    private function syncTemplates()
    {
        try {
            $templates = $this->getClient()->getTemplates()->fetchTemplates();
        } catch (Exception\BadConnectionException $e) {
            return [];
        } catch (Exception\BadRequestException $e) {
            return [];
        } catch (Exception\BadResponseBodyException $e) {
            return [];
        }

        $existingTemplates = $this->fetchTemplates();

        $templateIds = array_map(
            function ($template) {
                return $template['id'];
            },
            $templates
        );

        $fields = ['media', 'layout', 'name', 'channel', 'text', 'button_label', 'button_url', 'status'];
        $formatData = function ($templateData) use ($fields) {
            $data = [];
            foreach ($fields as $field) {
                $data[$field] = isset($templateData[$field]) ? pSQL($templateData[$field]) : null;
            }

            return $data;
        };
        $isChanged = function ($old, $new) use ($fields) {
            $changed = false;
            foreach ($fields as $field) {
                if (!isset($new[$field])) {
                    continue;
                }

                if ($old[$field] !== (string)$new[$field]) {
                    $changed = true;
                    break;
                }
            }

            return $changed;
        };

        $forDeletion = $forUpdate = [];
        foreach ($existingTemplates as $template) {
            if (!in_array($template['provider_id'], $templateIds)) {
                $forDeletion[] = $template['id'];
            } elseif (false !== $index = array_search($template['provider_id'], array_column($templates, 'id'))) {
                if ($isChanged($template, $templates[$index])) {
                    $forUpdate[$template['id']] = $formatData($templates[$index]);
                }
            }
        }

        $fetchFresh = false;
        // Handle removals
        if (!empty($forDeletion)) {
            $fetchFresh = true;
            Db::getInstance()->delete('connectix_templates', 'id IN ('.pSQL(join(',', $forDeletion)).')');
        }

        // Handle updates
        foreach ($forUpdate as $id => $data) {
            $fetchFresh = true;
            Db::getInstance()->update('connectix_templates', $data, 'id = '.$id);
        }

        // Handle insertions
        foreach ($templates as $template) {
            if (false === array_search($template['id'], array_column($existingTemplates, 'provider_id'))) {
                $fetchFresh = true;
                $data = $formatData($template);
                $data['provider_id'] = $template['id'];

                try {
                    Db::getInstance()->insert('connectix_templates', $data);
                } catch (PrestaShopDatabaseException $e) {
                    continue;
                }
            }
        }

        // Fetch fresh data if it's necessary
        return $fetchFresh ? $this->fetchTemplates() : $existingTemplates;
    }

    /**
     * @param string|null $type
     * @param string|int|null $id
     * @return array
     */
    public static function fetchRules($type = null, $id = null)
    {
        $rules = [];
        $where = [];
        if ($type && in_array(
                $type,
                [self::TYPE_NEW_ORDER, self::TYPE_CHANGE_ORDER_STATUS, self::TYPE_READY_FOR_DELIVERY, self::TYPE_NOT_PICKED_UP]
            )) {
            $where[] = 'r.type = "'.pSQL($type).'"';
        }
        if ($id) {
            $where[] = 'r.id = "'.pSQL($id).'"';
        }
        if (!empty($where) && !$id) {
            $where[] = 'r.enabled = "1"';
        }

        try {
            $rules = Db::getInstance()->executeS(
                'SELECT
                    r.*,
                    rt.position as rt_position,
                    t.id as t_id,
                    t.name as t_name,
                    t.channel as t_channel,
                    t.status as t_status,
                    t.provider_id as t_provider_id
                FROM
                    '._DB_PREFIX_.'connectix_rules r
                LEFT JOIN '._DB_PREFIX_.'connectix_rules_templates rt ON rt.rule_id = r.id
                LEFT JOIN '._DB_PREFIX_.'connectix_templates t ON rt.template_id = t.id
                '.(!empty($where) ? 'WHERE '.join(' AND ', $where) : '').'
                ORDER BY rt.position ASC
            '
            );
        } catch (PrestaShopDatabaseException $e) {}

        if (!$rules) {
            return [];
        }

        $normalized = [];
        foreach ($rules as $rule) {
            $template = [
                'id' => $rule['t_id'],
                'position' => $rule['rt_position'],
                'name' => $rule['t_name'],
                'channel' => $rule['t_channel'],
                'status' => $rule['t_status'],
                'provider_id' => $rule['t_provider_id'],
            ];

            if (!isset($normalized[$rule['id']])) {
                $normalized[$rule['id']] = [
                    'id' => $rule['id'],
                    'fallback' => $rule['fallback'] === '1',
                    'type' => $rule['type'] !== '' ? $rule['type'] : null,
                    'order_status' => $rule['order_status'] !== '' ? $rule['order_status'] : null,
                    'delivery_type' => $rule['delivery_type'] !== '' ? $rule['delivery_type'] : null,
                    'days_not_picked_up' => $rule['days_not_picked_up'] !== '' ? $rule['days_not_picked_up'] : null,
                    'enabled' => '1' === $rule['enabled'],
                    'templates' => [$template],
                ];
            } else {
                $normalized[$rule['id']]['templates'][] = $template;
            }
        }

        return array_values($normalized);
    }

    /**
     * @return array
     */
    private function fetchOrderStatuses()
    {
        try {
            return array_map(
                function ($status) {
                    return [
                        'key' => $status['id_order_state'],
                        'value' => $status['id_order_state'],
                        'text' => $status['name'],
                    ];
                },
                Db::getInstance()->executeS(
                    'SELECT
                        os.id_order_state,
                        osl.name
                    FROM
                        '._DB_PREFIX_.'order_state os
                    INNER JOIN '._DB_PREFIX_.'order_state_lang osl ON os.id_order_state = osl.id_order_state AND osl.id_lang='.(int)$this->context->language->id.'
                    WHERE os.deleted = 0
                    ORDER BY osl.name ASC'
                )
            );
        } catch (PrestaShopDatabaseException $e) {
            return [];
        }
    }

    /**
     * @return array|null
     */
    private function saveRule()
    {
        $data = [];
        foreach (['enabled', 'fallback', 'type', 'delivery_type', 'order_status', 'days_not_picked_up'] as $field) {
            $value = pSQL(Tools::getValue($field));
            switch ($value) {
                case 'null':
                    $value = null;
                    break;
                case 'false':
                case 'true':
                    $value = $value === 'true' ? 1 : 0;
                    break;
            }
            $data[$field] = $value;
        }

        if (Tools::getIsset('id')) {
            $id = (int)Tools::getValue('id');
            Db::getInstance()->delete('connectix_rules_templates', 'rule_id = '.$id);
            Db::getInstance()->update('connectix_rules', $data, 'id = '.$id);
        } else {
            try {
                Db::getInstance()->insert('connectix_rules', $data);
                $id = Db::getInstance()->Insert_ID();
            } catch (PrestaShopDatabaseException $e) {
                return null;
            }
        }

        foreach (Tools::getValue('templates') as $i => $templateId) {
            try {
                Db::getInstance()->insert(
                    'connectix_rules_templates',
                    ['template_id' => $templateId, 'rule_id' => $id, 'position' => $i]
                );
            } catch (PrestaShopDatabaseException $e) {}
        }

        $record = $this->fetchRules(null, $id);
        if (!empty($record)) {
            $record = $record[0];
        }

        return $record;
    }

    /**
     * @return string
     */
    private function deleteRule()
    {
        if (Tools::getIsset('id')) {
            $id = (int)Tools::getValue('id');
            Db::getInstance()->delete('connectix_rules', 'id = '.$id);

            return "OK";
        }

        header($this->getProtocol()." 404 Not found");

        return "Not found";
    }

    /**
     * @param string $phone
     * @return bool
     */
    private function isPhoneBlacklisted(string $phone)
    {
        $blacklist = new Blacklist();
        $blacklist->setPhone($phone);
        try {
            $isBlacklisted = $this->getClient()->getBlacklist()->isBlacklisted($blacklist);
        } catch (Exception\BadConnectionException $e) {
            return false;
        } catch (Exception\BadRequestException $e) {
            return false;
        } catch (Exception\BadResponseBodyException $e) {
            return false;
        }

        return $isBlacklisted && $isBlacklisted === 'yes';
    }

    /**
     * @return mixed|string
     */
    private function getProtocol()
    {
        return (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0');
    }

    ##########################################################
    #                    Configuration Page                  #
    ##########################################################
    /**
     * @param $path
     * @param $token
     * @param array $data
     * @param array $params
     * @return false|mixed
     */
    private function call($path, $token, array $data = [], array $params = [])
    {
        if (empty($token)) {
            return false;
        }

        $headers = [
            "Content-Type: application/json",
            "Authorization: {$token}",
        ];

        $url = BaseRequest::SANDBOX_ENDPOINT;
        if (Configuration::get('CONNECTIX_SYSTEM') === self::SYSTEM_LIVE) {
            $url = BaseRequest::LIVE_ENDPOINT;
        }
        $url .= $path;
        if (!empty($params)) {
            $url .= '?'.http_build_query($params);
        }

        $ch = curl_init();

        curl_setopt_array($ch, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => false,
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_TIMEOUT => 3,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => json_encode($data),
        ]);

        $error = false;
        if (!$result = curl_exec($ch)) {
            $error = true;
        }

        if (200 !== curl_getinfo($ch, CURLINFO_RESPONSE_CODE) || 0 !== curl_errno($ch)) {
            $error = true;
        }

        curl_close($ch);

        if ($error) {
            return false;
        }

        return json_decode($result, true);
    }

    /**
     * @return bool|string
     */
    public function getContent()
    {
        if (Tools::isSubmit('ajax')) {
            switch (Tools::getValue('action')) {
                case 'getData':
                    $data = [
                        'settings' => $this->getSettings(),
                        'orderStatuses' => $this->fetchOrderStatuses(),
                        'templates' => $this->fetchTemplates(),
                        'rules' => $this->fetchRules(),
                    ];
                    break;
                case 'saveSettings':
                    $oldSettings = $this->getSettings(true);
                    foreach (array_keys($oldSettings) as $setting) {
                        $name = $setting;
                        if ('token' === $setting) {
                            $name = 'connectixToken';
                        }

                        if (Tools::getIsset($name)) {
                            Configuration::updateValue('CONNECTIX_'.strtoupper($this->fromCamelCase($setting)), Tools::getValue($name));
                        }
                    }

                    $newSettings = $this->getSettings(true);

                    $isTokenChanged = $oldSettings['token'] !== $newSettings['token'];
                    if ($isTokenChanged && $oldSettings['token'] && $oldSettings['integrationToken']) {
                        $data = new IntegrationUnregister();
                        $data->setSecret($oldSettings['secret'])
                            ->setToken($oldSettings['integrationToken']);
                        try {
                            $this->getClient()->setToken($oldSettings['token'])->getIntegration()->unregister($data);
                        } catch (\Exception $e) {}
                        Configuration::updateValue('CONNECTIX_INTEGRATION_TOKEN', '');
                    }

                    $templates = null;
                    if ($isTokenChanged && $newSettings['token']) {
                        $data = new Integration();
                        $data->setSecret($newSettings['secret'])
                            ->setPlatform('PrestaShop')
                            ->setType(Integration::TYPE_ECOMMERCE);
                        try {
                            $response = $this->getClient()->setToken($newSettings['token'])->getIntegration()->register($data);
                        } catch (\Exception $e) {
                            $response = null;
                        }

                        if ($response) {
                            Configuration::updateValue('CONNECTIX_INTEGRATION_TOKEN', $response['token']);
                            $templates = $this->syncTemplates();
                        }
                    }

                    $newSettings = $this->getSettings(true);
                    $this->unregisterHooks($oldSettings, $newSettings);
                    $this->registerHooks($oldSettings, $newSettings);

                    $data = $this->getSettings();
                    if ($templates) {
                        $data['templates'] = $templates;
                    }
                    break;
                case 'syncTemplates':
                    $data = $this->syncTemplates();
                    break;
                case 'saveRule':
                    $data = $this->saveRule();
                    break;
                case 'deleteRule':
                    $data = $this->deleteRule();
                    break;
                default:
                    $data = [];
                    break;
            }

            echo json_encode($data);
            exit;
        }

        $files = [];
        $manifest = $this->getLocalPath().'views/ui/asset-manifest.json';
        if (file_exists($manifest) && $manifestContent = Tools::file_get_contents($manifest)) {
            $json = json_decode($manifestContent, true);
            foreach ($json['entrypoints'] as $entrypoint) {
                if (false !== strpos($entrypoint, '.js')) {
                    $files[] = $this->_path . 'views/ui/'.$entrypoint;
                } else {
                    $this->context->controller->addCss($this->_path . 'views/ui/'.$entrypoint);
                }
            }
        }

        $this->smarty->assign([
            'link' => 'index.php?controller=AdminModules&token='.Tools::getAdminTokenLite('AdminModules').'&configure='.$this->name,
            'files' => $files,
        ]);

        return $this->display($this->getLocalPath(), 'views/templates/admin/connectix.tpl');
    }

    /**
     * @param array $oldSettings
     * @param array $newSettings
     */
    private function unregisterHooks(array $oldSettings, array $newSettings)
    {
        $shouldUnregister = $oldSettings['token'] !== $newSettings['token'] || ($oldSettings['enabled'] !== $newSettings['enabled'] && !$newSettings['enabled']);

        $client = $this->getClient()->setToken($oldSettings['token']);
        foreach ([IntegrationHook::TYPE_CALLBACK, IntegrationHook::TYPE_INBOUND, IntegrationHook::TYPE_TEMPLATE, IntegrationHook::TYPE_TRACKING] as $type) {
            $shouldUnregisterTrackingHook = $type === IntegrationHook::TYPE_TRACKING && $oldSettings['trackDelivery'] !== $newSettings['trackDelivery'] && !$newSettings['trackDelivery'];
            if ($shouldUnregister || $shouldUnregisterTrackingHook) {
                $hookSettingsName = 'hook'.ucfirst($type);
                $hookId = $newSettings[$hookSettingsName];
                if (!empty($hookId) && $oldSettings['token']) {
                    $data = new IntegrationHookUnregister();
                    $data->setId($hookId);
                    try {
                        $client->getIntegrationHook()->unregister($data);
                    } catch (\Exception $e) {}
                    Configuration::updateValue('CONNECTIX_'.strtoupper($this->fromCamelCase($hookSettingsName)), '');
                }
            }
        }
    }

    /**
     * @param array $oldSettings
     * @param array $newSettings
     */
    private function registerHooks(array $oldSettings, array $newSettings)
    {
        // Hooks callback|inbound|template|tracking
        $token = $newSettings['token'];
        $secret = $newSettings['secret'];
        $integrationToken = $newSettings['integrationToken'];

        $shouldRegister = $integrationToken && (($oldSettings['enabled'] !== $newSettings['enabled'] && $newSettings['enabled']) || ($oldSettings['token'] !== $token && $token && $newSettings['enabled']));

        $client = $this->getClient()->setToken($token);
        foreach ([IntegrationHook::TYPE_CALLBACK, IntegrationHook::TYPE_INBOUND, IntegrationHook::TYPE_TEMPLATE, IntegrationHook::TYPE_TRACKING] as $type) {
            $shouldRegisterTrackingHook = $integrationToken && $newSettings['enabled'] && $type === IntegrationHook::TYPE_TRACKING && $oldSettings['trackDelivery'] !== $newSettings['trackDelivery'] && $newSettings['trackDelivery'];
            if (($type !== IntegrationHook::TYPE_TRACKING && $shouldRegister) || $shouldRegisterTrackingHook) {
                $hookSettingsName = 'hook'.ucfirst($type);
                $url = Context::getContext()->link->getModuleLink('connectix', 'callback', [
                    'ajax' => 1,
                    'type' => $type,
                    'secret' => $secret,
                ]);
                $data = new IntegrationHook();
                $data->setType($type)
                    ->setToken($integrationToken)
                    ->setUrl($url);

                try {
                    $response = $client->getIntegrationHook()->register($data);
                    Configuration::updateValue('CONNECTIX_'.strtoupper($this->fromCamelCase($hookSettingsName)), $response['id']);
                } catch (\Exception $e) {}
            }
        }
    }

    /**
     * @return string
     */
    private function generateOrderToken()
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < 32; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    /**
     * @param $amount
     * @param Currency|null $currency
     * @return mixed
     */
    private function formatPrice($amount, $currency = null)
    {
        $context = Context::getContext();
        if (!$currency) {
            $currency = $context->currency;
        }

        if (version_compare(_PS_VERSION_, '1.7.6', '>=')) {
            return Tools::getContextLocale($context)->formatPrice($amount, $currency->iso_code);
        }

        return Tools::displayPrice($amount, $currency);
    }

    ##########################################################
    #                          Hooks                         #
    ##########################################################
    /**
     * @return bool|string
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function hookDisplayAdminOrderSideBottom()
    {
        $id_order = (int)Tools::getValue('id_order');

        try {
            $records = Db::getInstance()->executeS(
                'SELECT
                    m.*,
                    mi.id as mi_id,
                    mi.message_id as mi_message_id,
                    mi.type as mi_type,
                    mi.text as mi_text,
                    mi.media_url as mi_media_url,
                    mi.media_name as mi_media_name,
                    mi.sent_at as mi_sent_at
                FROM '._DB_PREFIX_.'connectix_messages m
                LEFT JOIN '._DB_PREFIX_.'connectix_message_inbounds mi ON m.id = mi.message_id
                WHERE
                    m.status <> \'pending\'
                AND
                    m.order_id = '.(int)$id_order.'
                ORDER BY m.id ASC'
            );
        } catch (PrestaShopDatabaseException $e) {
            return '';
        }

        $viberLink = null;
        $messages = [];

        foreach ($records as $record) {
            if (!$viberLink && $record['channel'] === 'viber') {
                $order = new Order($id_order);
                $address = new Address($order->id_address_delivery);
                $country = new Country($address->id_country);
                $phone = $address->phone_mobile;

                if ('08' === Tools::substr($phone, 0, 2)) {
                    $phone = '+'.$country->call_prefix.ltrim($phone, '0');
                }
                $viberLink = 'viber://chat?number='.$phone;
            }

            if (!isset($messages[$record['id']])) {
                $messages[$record['id']] = [
                    "id" => $record['id'],
                    "order_id" => $record['order_id'],
                    "fallback_id" => $record['fallback_id'],
                    "type" => $record['type'],
                    "text" => $record['text'],
                    "channel" => $record['channel'],
                    "status" => 'message_status_'.$record['status'],
                    "created_at" => $record['created_at'],
                    "provider_id" => $record['provider_id'],
                    "inbounds" => [],
                ];
            }

            if (!empty($record['mi_id'])) {
                $messages[$record['id']]['inbounds'][] = [
                    'id' => $record['mi_id'],
                    'type' => $record['mi_type'],
                    'text' => $record['mi_text'],
                    'media_url' => $record['mi_media_url'],
                    'media_name' => $record['mi_media_name'],
                    'sent_at' => $record['mi_sent_at'],
                ];
            }
        }

        $this->context->smarty->assign([
            'messages' => array_values($messages),
            'viberLink' => $viberLink,
        ]);

        return $this->display(__FILE__, '/views/templates/hook/display_order_messages.tpl');
    }

    private function getDefaultParameters($order)
    {
        $shippingAddress = new Address($order->id_address_delivery);

        $products = $order->getProducts();

        $productsAmount = (float)$order->total_products_wt;
        $shippingAmount = (float)$order->total_shipping_tax_incl;
        $discountAmount = (float)$order->total_discounts_tax_incl;
        $totalAmount = $productsAmount + $shippingAmount - $discountAmount;
        $productsCount = 0;

        $currency = new Currency($order->id_currency);
        $carrier = new Carrier($order->id_carrier);

        $formattedProducts = [];
        foreach ($products as $product) {
            $productName = $product['product_name'];
            if (50 < mb_strlen($productName)) {
                $productName = trim(mb_substr($productName, 0, 27)).' ... '.trim(mb_substr($productName, -20));
            }

            $quantity = (int)$product['product_quantity'];
            $price = $this->formatPrice($product['total_price_tax_incl'], $currency);

            $productsCount += $quantity;
            $formattedProducts[] = "{$productName} x {$quantity} = {$price}";
        }

        $translator = Context::getContext()->getTranslator();
        $totals = [
            $translator->trans('Subtotal', array(), 'Shop.Theme.Checkout').": ".$this->formatPrice($productsAmount, $currency),
        ];

        if (0.01 < $discountAmount) {
            $totals[] = $translator->trans('Discount', array(), 'Shop.Theme.Checkout').": ".$this->formatPrice($discountAmount, $currency);
        }
        $totals[] = $translator->trans('Shipping', array(), 'Shop.Theme.Checkout').": ".$this->formatPrice($shippingAmount, $currency);
        $totals[] = $translator->trans('Total', array(), 'Shop.Theme.Checkout').": ".$this->formatPrice($totalAmount, $currency);

        return [
            'order_number' => (string)$order->id,
            'first_name' => (string)$shippingAddress->firstname,
            'last_name' => (string)$shippingAddress->lastname,
            'total_amount' => $this->formatPrice($totalAmount, $currency),
            'products' => join("\n\n", $formattedProducts),
            'totals' => join("\n", $totals),
            'payment_method' => $order->payment,
            'delivery_type' => $carrier->name,
            'delivery_address' => $shippingAddress->city . ', '. $shippingAddress->address1,
            'products_count' => (string)$productsCount,
        ];
    }

    /**
     * @param array $params
     */
    public function hookActionValidateOrder(array $params)
    {
        if (!(int)Configuration::get('CONNECTIX_ENABLED')) {
            return;
        }

        /** @var Order $order */
        $order = $params['order'];
        $token = null;
        do {
            $tmp = $this->generateOrderToken();

            if (!(int)Db::getInstance()->getValue('SELECT COUNT(*) FROM '._DB_PREFIX_.'connectix_order_token WHERE token = "'.pSQL($tmp).'"')) {
                $token = $tmp;
            }
        } while(is_null($token));

        $now = (new DateTime('now'))->format('Y-m-d H:i:s');
        try {
            Db::getInstance()->insert('connectix_order_token', [
                'order_id' => pSQL($order->id),
                'token' => pSQL($token),
                'created_at' => $now,
                'updated_at' => $now,
            ]);
        } catch (PrestaShopDatabaseException $e) {}

        $rules = $this->fetchRules(self::TYPE_NEW_ORDER);
        if (empty($rules)) {
            return;
        }

        $address = new Address($order->id_address_delivery);
        $phone = $address->phone_mobile;
        if ($this->isPhoneBlacklisted($phone)) {
            return;
        }

        $parameters = $this->getDefaultParameters($order);

        $contact = new Contact();
        $contact->setFirstName($parameters['first_name'])
            ->setLastName($parameters['last_name']);

        foreach (['confirm', 'cancel'] as $action) {
            $parameters[$action.'_url'] = '';

            $actionUp = strtoupper($action);
            if ((int)Configuration::get('CONNECTIX_'.$actionUp.'_ORDER')) {
                $parameters[$action.'_url'] = Context::getContext()->link->getModuleLink('connectix', 'callback', [
                    'ajax' => 1,
                    'type' => $action,
                    'token' => $token,
                ]);
            }
        }

        foreach ($rules as $rule) {
            $this->sendMessage($order->id, $phone, $rule, $parameters, $contact);
        }
    }

    /**
     * @param array $params
     */
    public function hookActionOrderStatusPostUpdate(array $params)
    {
        if (!(int)Configuration::get('CONNECTIX_ENABLED')) {
            return;
        }

        $rules = array_filter($this->fetchRules(self::TYPE_CHANGE_ORDER_STATUS), function ($rule) use ($params) {
            return (int)$rule['order_status'] === (int)$params['newOrderStatus']->id;
        });
        if (empty($rules)) {
            return;
        }

        try {
            $order = new Order($params['id_order']);
            $address = new Address($order->id_address_delivery);
        } catch (PrestaShopDatabaseException $e) {
            return;
        } catch (PrestaShopException $e) {
            return;
        }

        $phone = $address->phone_mobile;
        if ($this->isPhoneBlacklisted($phone)) {
            return;
        }

        $parameters = $this->getDefaultParameters($order);
        $parameters['tracking_number'] = $order->getWsShippingNumber();

        $contact = new Contact();
        $contact->setFirstName($parameters['first_name'])
            ->setLastName($parameters['last_name']);

        foreach ($rules as $rule) {
            $this->sendMessage($params['id_order'], $phone, $rule, $parameters, $contact);
        }
    }

    /**
     * @param array $params
     */
    public function hookActionObjectOrderCarrierUpdateAfter(array $params)
    {
        if (!(int)Configuration::get('CONNECTIX_ENABLED') || !(int)Configuration::get('CONNECTIX_TRACK_DELIVERY')) {
            return;
        }

        /** @var OrderCarrier $order */
        $orderCarrier = $params['object'];
        $trackingNumber = $orderCarrier->tracking_number;

        if (empty($trackingNumber)) {
            return;
        }

        if (is_array($trackingNumber)) {
            $trackingNumber = $trackingNumber[0];
        }

        if (is_object($trackingNumber) && $trackingNumber instanceof SimpleXMLElement) {
            $trackingNumber = (string)$trackingNumber;
        }

        $data = [
            'trackingNumber' => $trackingNumber,
        ];
        $params = [
            'integration' => Configuration::get('CONNECTIX_INTEGRATION_TOKEN'),
        ];

        $this->call('trackings', Configuration::get('CONNECTIX_TOKEN'), $data, $params);

        $tracking = Db::getInstance()->getRow('SELECT id FROM '._DB_PREFIX_.'connectix_tracking WHERE order_id = "'.pSQL($orderCarrier->id_order).'" AND tracking_number = "'.pSQL($trackingNumber).'"');

        try {
            if ($tracking) {
                Db::getInstance()->update('connectix_tracking', [
                    'tracking_number' => $trackingNumber,
                ], 'id = '.(int)$tracking['id']);
            } else {
                Db::getInstance()->insert('connectix_tracking', [
                    'order_id' => $orderCarrier->id_order,
                    'tracking_number' => $trackingNumber,
                ]);
            }
        } catch (PrestaShopDatabaseException $e) {
        }
    }

    /**
     * @param array $params
     */
    public function hookConnectixReadyForDelivery(array $params)
    {
        if (!(int)Configuration::get('CONNECTIX_ENABLED')) {
            return;
        }

        $rules = $this->fetchRules(self::TYPE_READY_FOR_DELIVERY);
        if (empty($rules)) {
            return;
        }

        $doorRules = array_filter(
            $rules,
            function ($rule) {
                return 'door' === $rule['delivery_type'];
            }
        );
        $officeRules = array_filter(
            $rules,
            function ($rule) {
                return 'office' === $rule['delivery_type'];
            }
        );

        foreach ($params['orders'] as $tmp) {
            /** @var Order $order */
            $order = $tmp['order'];
            $deliveryData = $tmp['deliveryData'];

            $address = new Address($order->id_address_delivery);
            $phone = $address->phone_mobile;

            if ($this->isPhoneBlacklisted($phone)) {
                continue;
            }

            $parameters = $this->getDefaultParameters($order);
            $parameters['tracking_number'] = $order->getWsShippingNumber();

            $contact = new Contact();
            $contact->setFirstName($parameters['first_name'])
                ->setLastName($parameters['first_name']);

            if ('office' === $deliveryData['deliveryType']) {
                $parameters['office'] = $deliveryData['office'];
                foreach ($officeRules as $rule) {
                    $this->sendMessage($order->id, $phone, $rule, $parameters, $contact);
                }
            } elseif ('door' === $deliveryData['deliveryType']) {
                $parameters['courier'] = $deliveryData['courier'];
                foreach ($doorRules as $rule) {
                    $this->sendMessage($order->id, $phone, $rule, $parameters, $contact);
                }
            }
        }
    }

    /**
     * @param array $params
     */
    public function hookConnectixNotPickedUp(array $params)
    {
        if (!(int)Configuration::get('CONNECTIX_ENABLED')) {
            return;
        }

        foreach ($params['orders'] as $tmp) {
            /** @var Order $order */
            $order = $tmp['order'];
            $deliveryData = $tmp['deliveryData'];

            $address = new Address($order->id_address_delivery);
            $phone = $address->phone_mobile;

            if ($this->isPhoneBlacklisted($phone)) {
                continue;
            }

            $parameters = $this->getDefaultParameters($order);
            $parameters['tracking_number'] = $order->getWsShippingNumber();
            $parameters['office'] = $deliveryData['office'];
            $parameters['days_not_picked_up'] = (string)$deliveryData['workingDaysPassed'];

            $contact = new Contact();
            $contact->setFirstName($parameters['first_name'])
                ->setLastName($parameters['last_name']);

            foreach ($params['rules'] as $rule) {
                $this->sendMessage($order->id, $phone, $rule, $parameters, $contact);
            }
        }
    }
}
