<?php

namespace Connectix\Request;

use Connectix\Components\ComponentInterface;
use Connectix\Exception;

/**
 * Class BaseRequest
 * @package Connectix\Request
 */
abstract class BaseRequest implements RequestInterface
{
	const LIVE_ENDPOINT = 'https://api.connectix.bg/';
	const SANDBOX_ENDPOINT = 'https://api-sandbox.connectix.bg/';

	/**
	 * @var string
	 */
	private $token;

	/**
	 * @var bool
	 */
	private $isDev;

	/**
	 * @param string $token
	 * @param bool $isDev
	 */
	public function __construct($token, $isDev = true)
	{
		$this->token = $token;
		$this->isDev = $isDev;
	}

	/**
	 * @return string
	 */
	public function getToken()
	{
		return $this->token;
	}

	/**
	 * @param string $token
	 * @return BaseRequest
	 */
	public function setToken($token)
	{
		$this->token = $token;

		return $this;
	}

	/**
	 * @return bool
	 */
	public function isDev()
	{
		return $this->isDev;
	}

	/**
	 * @param bool $isDev
	 * @return BaseRequest
	 */
	public function setIsDev($isDev)
	{
		$this->isDev = $isDev;

		return $this;
	}

	/**
	 * @return string
	 */
	abstract protected function getBasePath();

	/**
	 * @return string
	 */
	private function getEndpoint()
	{
		return $this->isDev ? self::SANDBOX_ENDPOINT : self::LIVE_ENDPOINT;
	}

	/**
	 * @param string|null $uri
	 * @param ComponentInterface|array|null $data
	 * @param array $query
	 * @return array|string
	 * @throws Exception\BadConnectionException
	 * @throws Exception\BadRequestException
	 * @throws Exception\BadResponseBodyException
	 */
	public function call($uri = null, $data = null, $query = [])
	{
		$headers = [
			"Content-Type: application/json",
			"Authorization: {$this->token}",
		];

		$url = $this->getEndpoint().$this->getBasePath();
		if ($uri) {
			$url .= $uri;
		}
		if (!empty($query)) {
			$url .= '?' . http_build_query($query);
		}

		$ch = curl_init();
		$options = [
			CURLOPT_URL => $url,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_FOLLOWLOCATION => false,
			CURLOPT_HTTPHEADER => $headers,
			CURLOPT_TIMEOUT => 15
		];

		if ($data instanceof ComponentInterface) {
			$options[CURLOPT_POST] = true;
			$options[CURLOPT_POSTFIELDS] = $data->toJson();
		} elseif (is_array($data)) {
			$options[CURLOPT_POST] = true;
			$options[CURLOPT_POSTFIELDS] = json_encode($data);
		}

		curl_setopt_array($ch, $options);
		$response = curl_exec($ch);

		if (!curl_errno($ch)) {
			$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			switch ($httpCode) {
				case 200:
					break;
				default:
					throw new Exception\BadRequestException($httpCode, $response);
			}
		} else {
			throw new Exception\BadConnectionException();
		}

		curl_close($ch);

		$content = json_decode($response, true);

		if (null === $content && json_last_error() !== JSON_ERROR_NONE) {
			throw new Exception\BadResponseBodyException();
		}

		return $content;
	}
}
